@@ -2,9 +2,15 @@
 from __future__ import unicode_literals
 import sys
 import unittest
-from curtsies.formatstring import (FmtStr, fmtstr, Chunk, linesplit,
-                                   normalize_slice, width_aware_slice)
-from curtsies.fmtfuncs import *
+from curtsies.formatstring import (
+    FmtStr,
+    fmtstr,
+    Chunk,
+    linesplit,
+    normalize_slice,
+    width_aware_slice,
+)
+from curtsies.fmtfuncs import (blue, red, green, on_blue, on_red, on_green, underline, blink, bold)
 from curtsies.termformatconstants import FG_COLORS
 from curtsies.formatstringarray import fsarray, FSArray, FormatStringTest
 
@@ -59,8 +65,8 @@ class TestFmtStrParsing(unittest.TestCase):
         self.assertEqual(str(fmtstr('\x1b[43mhello\x1b[0m')), '\x1b[43mhello\x1b[49m')
         self.assertEqual(str(fmtstr('\x1b[43mhello')), '\x1b[43mhello\x1b[49m')
         self.assertEqual(str(fmtstr('\x1b[32;1mhello')), '\x1b[32m\x1b[1mhello\x1b[0m\x1b[39m')
-        self.assertEqual(str(fmtstr('\x1b[2mhello')), 'hello')
-        self.assertEqual(str(fmtstr('\x1b[32;2mhello')), '\x1b[32mhello\x1b[39m')
+        self.assertEqual(str(fmtstr('\x1b[2mhello')), '\x1b[2mhello\x1b[0m')
+        self.assertEqual(str(fmtstr('\x1b[32;2mhello')), '\x1b[32m\x1b[2mhello\x1b[0m\x1b[39m')
         self.assertEqual(str(fmtstr('\x1b[33m\x1b[43mhello\x1b[0m')),
                          '\x1b[33m\x1b[43mhello\x1b[49m\x1b[39m')
 
@@ -112,7 +118,7 @@ class TestFmtStrSplice(unittest.TestCase):
     def test_splice_of_empty_fmtstr(self):
         self.assertEqual(fmtstr('ab').splice('', 1), fmtstr('ab'))
 
-    def test_splice_with_multiple_basefmtstrs(self):
+    def test_splice_with_multiple_chunks(self):
         a = fmtstr('notion')
         b = a.splice('te', 2, 6)
         c = b.splice('de', 0)
@@ -120,7 +126,7 @@ class TestFmtStrSplice(unittest.TestCase):
         self.assertEqual(a.s, "notion")
         self.assertEqual(b.s, "note")
         self.assertEqual(c.s, "denote")
-        self.assertEqual(len(c.basefmtstrs), 3)
+        self.assertEqual(len(c.chunks), 3)
 
     def test_splice_fmtstr_with_end_without_atts(self):
         a = fmtstr('notion')
@@ -128,36 +134,36 @@ class TestFmtStrSplice(unittest.TestCase):
 
         self.assertEqual(a.s, "notion")
         self.assertEqual(b.s, "note")
-        self.assertEqual(len(b.basefmtstrs), 2)
+        self.assertEqual(len(b.chunks), 2)
 
     def test_splice_fmtstr_with_end_with_atts(self):
-        # Need to test with fmtstr consisting of multiple basefmtstrs
+        # Need to test with fmtstr consisting of multiple chunks
         # and with attributes
         a = fmtstr('notion', 'blue')
         b = a.splice('te', 2, 6)
 
         self.assertEqual(a.s, "notion")
-        self.assertEqual(a.basefmtstrs[0].atts, {'fg': 34})
-        self.assertEqual(len(a.basefmtstrs), 1)
+        self.assertEqual(a.chunks[0].atts, {'fg': 34})
+        self.assertEqual(len(a.chunks), 1)
 
         self.assertEqual(b.s, 'note')
-        self.assertEqual(b.basefmtstrs[0].atts, {'fg': 34})
-        self.assertEqual(b.basefmtstrs[1].atts, {})
-        self.assertEqual(len(b.basefmtstrs), 2)
+        self.assertEqual(b.chunks[0].atts, {'fg': 34})
+        self.assertEqual(b.chunks[1].atts, {})
+        self.assertEqual(len(b.chunks), 2)
 
     def test_splice_fmtstr_without_end(self):
         a = fmtstr('notion')
         b = a.splice(fmtstr('ta'), 2)
         self.assertEqual(a.s, 'notion')
         self.assertEqual(b.s, 'notation')
-        self.assertEqual(len(b.basefmtstrs), 3)
+        self.assertEqual(len(b.chunks), 3)
 
     def test_splice_string_without_end(self):
         a = fmtstr('notion')
         b = a.splice('ta', 2)
         self.assertEqual(a.s, 'notion')
         self.assertEqual(b.s, 'notation')
-        self.assertEqual(len(b.basefmtstrs), 3)
+        self.assertEqual(len(b.chunks), 3)
 
     def test_multiple_bfs_splice(self):
         self.assertEqual(fmtstr('a') + blue('b'),
@@ -185,14 +191,14 @@ class TestFmtStr(unittest.TestCase):
         b = a.copy_with_new_str('bye')
         self.assertEqual(a.s, 'hello')
         self.assertEqual(b.s, 'bye')
-        self.assertEqual(a.basefmtstrs[0].atts, b.basefmtstrs[0].atts)
+        self.assertEqual(a.chunks[0].atts, b.chunks[0].atts)
 
     def test_append_without_atts(self):
         a = fmtstr('no')
         b = a.append('te')
         self.assertEqual(a.s, 'no')
         self.assertEqual(b.s, 'note')
-        self.assertEqual(len(b.basefmtstrs), 2)
+        self.assertEqual(len(b.chunks), 2)
 
     def test_shared_atts(self):
         a = fmtstr('hi', 'blue')
@@ -218,9 +224,9 @@ class TestFmtStr(unittest.TestCase):
         s = blue('hello there')
         self.assertEqual(s.split(' '), [s[:5], s[6:]])
 
-        # split shouldn't create fmtstrs without basefmtstrs
-        self.assertEqual(fmtstr('a').split('a')[0].basefmtstrs, fmtstr('').basefmtstrs)
-        self.assertEqual(fmtstr('a').split('a')[1].basefmtstrs, fmtstr('').basefmtstrs)
+        # split shouldn't create fmtstrs without chunks
+        self.assertEqual(fmtstr('a').split('a')[0].chunks, fmtstr('').chunks)
+        self.assertEqual(fmtstr('a').split('a')[1].chunks, fmtstr('').chunks)
 
         self.assertEqual((fmtstr('imp') + ' ').split('i'), [fmtstr(''), fmtstr('mp') + ' '])
         self.assertEqual(blue('abcbd').split('b'), [blue('a'), blue('c'), blue('d')])
@@ -422,6 +428,19 @@ class TestCharacterWidth(unittest.TestCase):
         self.assertEqual(fmtstr('HＥ!', 'blue').width_aware_slice(slice(1, 2, None)), fmtstr(' ', 'blue'))
         self.assertEqual(fmtstr('HＥ!', 'blue').width_aware_slice(slice(1, 3, None)), fmtstr('Ｅ', 'blue'))
 
+    def test_width_aware_splitlines(self):
+        s = fmtstr('abcd')
+        self.assertEqual(list(s.width_aware_splitlines(2)), [fmtstr('ab'), fmtstr('cd')])
+
+        s = fmtstr('HＥ!')
+        self.assertEqual(list(s.width_aware_splitlines(2)), [fmtstr('H '), fmtstr('Ｅ'), fmtstr('!')])
+
+        s = fmtstr('He\u0300llo')
+        self.assertEqual(list(s.width_aware_splitlines(2)), [fmtstr('He\u0300'), fmtstr('ll'), fmtstr('o')])
+
+        with self.assertRaises(ValueError):
+            s.width_aware_splitlines(1)
+
     def test_width_at_offset(self):
         self.assertEqual(fmtstr('abＥcdef').width_at_offset(0), 0)
         self.assertEqual(fmtstr('abＥcdef').width_at_offset(2), 2)
@@ -433,6 +452,7 @@ class TestCharacterWidth(unittest.TestCase):
         self.assertEqual(len(fmtstr('a\u0300')), 2)
         self.assertEqual(fmtstr('a\u0300').width, 1)
 
+
 class TestWidthHelpers(unittest.TestCase):
 
     def test_combining_char_aware_slice(self):
@@ -443,6 +463,8 @@ class TestWidthHelpers(unittest.TestCase):
         self.assertEqual(width_aware_slice('ab\u0300c', 0, 2), 'ab\u0300')
         self.assertEqual(width_aware_slice('ab\u0300c', 1, 3), 'b\u0300c')
         self.assertEqual(width_aware_slice('ab\u0300\u0300c', 1, 3), 'b\u0300\u0300c')
+        self.assertEqual(width_aware_slice('ab\u0300\u0300c', 0, 2), 'ab\u0300\u0300')
+        self.assertEqual(width_aware_slice('ab\u0300\u0300c', 2, 3), 'c')
 
     def test_char_width_aware_slice(self):
         self.assertEqual(width_aware_slice('abc', 1, 2), 'b')
@@ -452,6 +474,53 @@ class TestWidthHelpers(unittest.TestCase):
         self.assertEqual(width_aware_slice('aＥbc', 0, 2), 'a ')
 
 
+class TestChunk(unittest.TestCase):
+    def test_repr(self):
+        c = Chunk('a', {'fg': 32})
+        if PY2:
+            self.assertEqual(repr(c), """Chunk(u'a', {'fg': 32})""")
+        else:
+            self.assertEqual(repr(c), """Chunk('a', {'fg': 32})""")
+
+
+class TestChunkSplitter(unittest.TestCase):
+    def test_chunk_splitter(self):
+        splitter = Chunk('asdf', {'fg': 32}).splitter()
+        self.assertEqual(splitter.request(1), (1, Chunk('a', {'fg': 32})))
+        self.assertEqual(splitter.request(4), (3, Chunk('sdf', {'fg': 32})))
+        self.assertEqual(splitter.request(4), None)
+
+    def test_reusing_same_splitter(self):
+        c = Chunk('asdf', {'fg': 32})
+        s1 = c.splitter()
+        self.assertEqual(s1.request(3), (3, Chunk('asd', {'fg': 32})))
+        s1.reinit(c)
+        self.assertEqual(s1.request(3), (3, Chunk('asd', {'fg': 32})))
+        s1.reinit(c)
+        self.assertEqual(s1.request(3), (3, Chunk('asd', {'fg': 32})))
+
+        c2 = Chunk('abcdef', {})
+        s1.reinit(c2)
+        self.assertEqual(s1.request(3), (3, Chunk('abc')))
+
+    def test_width_awareness(self):
+        s = Chunk('asdf')
+        self.assertEqual(Chunk('ab\u0300c').splitter().request(3), (3, Chunk('ab\u0300c')))
+        self.assertEqual(Chunk('ab\u0300c').splitter().request(2), (2, Chunk('ab\u0300')))
+
+        s = Chunk('ab\u0300c').splitter()
+        self.assertEqual(s.request(1), (1, Chunk('a')))
+        self.assertEqual(s.request(2), (2, Chunk('b\u0300c')))
+
+        c = Chunk('aＥbc')
+        self.assertEqual(c.splitter().request(4), (4, Chunk('aＥb')))
+        s = c.splitter()
+        self.assertEqual(s.request(2), (2, Chunk('a ')))
+        self.assertEqual(s.request(2), (2, Chunk('Ｅ')))
+        self.assertEqual(s.request(2), (2, Chunk('bc')))
+        self.assertEqual(s.request(2), None)
+
+
 class TestFSArray(unittest.TestCase):
     def test_no_hanging_space(self):
         a = FSArray(4, 2)
